#! /usr/bin/perl
#
#    LCEnv.pl - Creates and syncs a complete SAPDB make environment.
#
#    @(#)LCEnv     2003-04-22
#
#    U. Jansen, SAP AG
#
#    ========== licence begin LGPL
#    Copyright (C) 2002 SAP AG
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#    ========== licence end
#

##############################################################################
# Static includes:

use strict;

use Getopt::Long;
use IO::SimpleCfg;
use IO::FilePath;
use Env::LCEnv;

##############################################################################
# Console Preparation (redirect STDERR to STDOUT and disable buffering):

close   STDERR;
open    STDERR, ">&STDOUT";
$| = 1;

##############################################################################
# Global variables:

my $ProgramName     = "LCEnv";
my $Version         = "0";
my $Revision        = "09";
my $Date            = "2003-05-07";

my %Opts            = ();
my $Glob            = {};

$Glob->{'CFG'}      = {};

##############################################################################
# Main routine:
##############################################################################

unless ($ENV{'TOOL'})
{
    print "%TOOL% is not set in the environment!\n";
    exit (10);
}

print "\n$ProgramName v$Version.$Revision ($Date), (c)2003 SAP AG\n\n";

# Check command line parameters for correctness:

if (!GetOptions( \%Opts,
                'h|help|?',
                'v|verbose',
                'y|yes',
                'b|batch',
                'd|default',
                'rs|resync',
                'own=s',
                'rel=s',
                'cl=i',
                'p4u=s',
                'p4c=s',
                'p4r=s',
                'prod',
                'm=s',
                'cfg=s',
                's|scratch',
                'c|clear',
                'rm|remove') || $Opts{'h'} || @ARGV)
{
    Usage();
}

# First, load the configuration file:
if ($Opts{'cfg'})
{
    $Glob->{'CFG'} = LoadSimpleCfg($Opts{'cfg'});
}
elsif ($Opts{'own'} && -d $Opts{'own'})
{
    my $cfg_file = JoinPath($Opts{'own'}, "LCEnv.cfg");
    if (-e $cfg_file)
    {
        $Glob->{'CFG'} = LoadSimpleCfg($cfg_file);
    }
}

# Then, overload settings by given command line parameters:
ParseCmdLineParams();

if ($Glob->{'CFG'}->{'ACTIONS'}->{'REMOVE'})
{
    #=========================================================================
    # REMOVE INSTANCE
    #=========================================================================

    LCERemEnv($Glob->{'CFG'});
}
else
{
    #=========================================================================
    # INSTALL INSTANCE
    #=========================================================================

    #-------------------------------------------------------------------------
    # First, check configuration integrity and ask for missing parameters:
    #-------------------------------------------------------------------------
    LCECheckEnv($Glob->{'CFG'});

    #-------------------------------------------------------------------------
    # Print out generated configuration:
    #-------------------------------------------------------------------------
    print "\nThe following configuration will be used:\n\n";

    ShowHashRef($Glob->{'CFG'});

    unless ($Glob->{'CFG'}->{'GLOBAL'}->{'ASSUME_YES'} || $Glob->{'CFG'}->{'GLOBAL'}->{'BATCH_MODE'} || (RequestInput("Is this correct (Y/n)?", "y", "[yYnN]", "Input incorrect. Please, use just [y or n].\n") =~ /^y$/i))
    {
        print "Aborting...\n";
        exit (0);
    }

    print "\n";

    #-------------------------------------------------------------------------
    # Now test / create / update the environment:
    #-------------------------------------------------------------------------
    LCECreateEnv($Glob->{'CFG'});

    #-------------------------------------------------------------------------
    # Save the generated configuration including all changes made:
    #-------------------------------------------------------------------------
    SaveSimpleCfg(JoinPath($Glob->{'CFG'}->{'RELEASE'}->{'OWN'},"LCEnv.cfg"), $Glob->{'CFG'});

    #-------------------------------------------------------------------------
    # Check for CLEAR flag and eventually remove everything again:
    #-------------------------------------------------------------------------
    if ($Glob->{'CFG'}->{'ACTIONS'}->{'CLEAR'})
    {
        LCERemEnv($Glob->{'CFG'});
    }
}

##############################################################################
# ParseCmdLineParams() - Checks and copies the command line parameters.
##############################################################################

sub ParseCmdLineParams
{
    # Global options:
    $Glob->{'CFG'}->{'GLOBAL'}->{'VERBOSE'}         = $Opts{'v'}    ? 1 : ($Glob->{'CFG'}->{'GLOBAL'}->{'VERBOSE'} ? 1 : 0);
    $Glob->{'CFG'}->{'GLOBAL'}->{'ASSUME_YES'}      = $Opts{'y'}    ? 1 : ($Glob->{'CFG'}->{'GLOBAL'}->{'ASSUME_YES'} ? 1 : 0);
    $Glob->{'CFG'}->{'GLOBAL'}->{'BATCH_MODE'}      = $Opts{'b'}    ? 1 : ($Glob->{'CFG'}->{'GLOBAL'}->{'BATCH_MODE'} ? 1 : 0);
    $Glob->{'CFG'}->{'GLOBAL'}->{'USE_DEFAULT'}     = $Opts{'d'}    ? 1 : ($Glob->{'CFG'}->{'GLOBAL'}->{'USE_DEFAULT'} ? 1 : 0);
    $Glob->{'CFG'}->{'GLOBAL'}->{'PROD_MAKE'}       = $Opts{'prod'} ? 1 : ($Glob->{'CFG'}->{'GLOBAL'}->{'PROD_MAKE'} ? 1 : 0);
    $Glob->{'CFG'}->{'GLOBAL'}->{'SCRATCH'}         = $Opts{'s'}    ? 1 : ($Glob->{'CFG'}->{'GLOBAL'}->{'SCRATCH'} ? 1 : 0);

    # Release specific options:
    $Glob->{'CFG'}->{'RELEASE'}->{'OWN'}            = $Opts{'own'}      ? $Opts{'own'} : ($Glob->{'CFG'}->{'RELEASE'}->{'OWN'} ? $Glob->{'CFG'}->{'RELEASE'}->{'OWN'} : "");
    $Glob->{'CFG'}->{'RELEASE'}->{'VERSION'}        = $Opts{'rel'}      ? $Opts{'rel'} : ($Glob->{'CFG'}->{'RELEASE'}->{'VERSION'} ? $Glob->{'CFG'}->{'RELEASE'}->{'VERSION'} : "");
    $Glob->{'CFG'}->{'RELEASE'}->{'CHANGELIST'}     = ($Opts{'cl'} > 0) ? $Opts{'cl'}  : ($Glob->{'CFG'}->{'RELEASE'}->{'CHANGELIST'} ? $Glob->{'CFG'}->{'RELEASE'}->{'CHANGELIST'} : 0);

    # Perforce specific options:
    $Glob->{'CFG'}->{'PERFORCE'}->{'USER'}          = $Opts{'p4u'} ? $Opts{'p4u'} : ($Glob->{'CFG'}->{'PERFORCE'}->{'USER'} ? $Glob->{'CFG'}->{'PERFORCE'}->{'USER'} : "");
    $Glob->{'CFG'}->{'PERFORCE'}->{'CLIENT'}        = $Opts{'p4c'} ? $Opts{'p4c'} : ($Glob->{'CFG'}->{'PERFORCE'}->{'CLIENT'} ? $Glob->{'CFG'}->{'PERFORCE'}->{'CLIENT'} : "");
    $Glob->{'CFG'}->{'PERFORCE'}->{'ROOT'}          = $Opts{'p4r'} ? $Opts{'p4r'} : ($Glob->{'CFG'}->{'PERFORCE'}->{'ROOT'} ? $Glob->{'CFG'}->{'PERFORCE'}->{'ROOT'} : "");
    $Glob->{'CFG'}->{'PERFORCE'}->{'SERVER_PATH'}   = $Glob->{'CFG'}->{'PERFORCE'}->{'SERVER_PATH'} ? $Glob->{'CFG'}->{'PERFORCE'}->{'SERVER_PATH'} : "";
    $Glob->{'CFG'}->{'PERFORCE'}->{'CLIENT_PATH'}   = $Glob->{'CFG'}->{'PERFORCE'}->{'CLIENT_PATH'} ? $Glob->{'CFG'}->{'PERFORCE'}->{'CLIENT_PATH'} : "";
    $Glob->{'CFG'}->{'PERFORCE'}->{'RESYNC'}        = $Opts{'rs'}  ? 1 : ($Glob->{'CFG'}->{'PERFORCE'}->{'RESYNC'} ? 1 : 0);

    # Actions to be performed:
    $Glob->{'CFG'}->{'ACTIONS'}->{'REMOVE'}         = $Opts{'rm'} ? 1 : ($Glob->{'CFG'}->{'ACTIONS'}->{'REMOVE'} ? 1 : 0);
    $Glob->{'CFG'}->{'ACTIONS'}->{'CLEAR'}          = $Opts{'c'}  ? 1 : ($Glob->{'CFG'}->{'ACTIONS'}->{'CLEAR'} ? 1 : 0);
    $Glob->{'CFG'}->{'ACTIONS'}->{'TARGET'}         = $Opts{'m'}  ? $Opts{'m'} : ($Glob->{'CFG'}->{'ACTIONS'}->{'TARGET'} ? $Glob->{'CFG'}->{'ACTIONS'}->{'TARGET'} : "");
    $Glob->{'CFG'}->{'ACTIONS'}->{'MAKE'}           = ($Glob->{'CFG'}->{'GLOBAL'}->{'PROD_MAKE'} || $Glob->{'CFG'}->{'ACTIONS'}->{'TARGET'}) ? 1 : 0;

    if ($Glob->{'CFG'}->{'ACTIONS'}->{'PROD_MAKE'} && $Glob->{'CFG'}->{'ACTIONS'}->{'TARGET'})
    {
        print "WARNING! Setting a specific target in productive mode is prohibited.\nThe target '$Glob->{'CFG'}->{'ACTIONS'}->{'TARGET'}' will be ignored.\n\n";
        $Glob->{'CFG'}->{'ACTIONS'}->{'TARGET'} = "";
    }
}


##############################################################################
# Usage() - Shows the program usage and exits the script.
##############################################################################

sub Usage
{
    print <<USAGE_END;

Usage: LCEnv.pl [-(h|help)] [-(v|verbose)] [-(y|yes)] [-(rs|resync)]
                [-own <OWN_DIR>] [-rel <RELEASE>] [-cl <CHANGELIST>]
                [-p4u <P4USER>] [-p4c <P4CLIENT>] [-p4r <P4ROOT>]
                [-prod] [-m <MAKE_TARGET>] [-cfg <CONFIG_FILE>]
                [-(s|scratch)] [-(c|clear)] [-(rm|remove)] [-(b|batch)]
                [-(d|default)]

Where:
        -h or -help or -?
            Displays this help file and ends program execution.

        -v or -verbose
            Produces a more detailed output (mostly for debugging).

        -y or -yes
            Assume 'yes' on all 'yes/no' questions.

        -rs or -resync
            Force a re-sync of all files.

        -own <OWN_DIR>
            Sets the new OWN directory to be used.

        -rel <RELEASE>
            Sets the release to be installed.
            This release must be given in one of the following formats:

            - 7403(DEV|COR|DEVELOP)
            - 740320
            - 7.4.03.(dev|cor|develop)
            - 7.4.03.20

            NOTE:
            1. If the release string is incomplete or missing, any
               missing parts will be asked for. You have then the
               opportunity to browse the perforce tree.
            2. There won't be a check for the given parts (except
               for the build prefix). If you give an incorrect
               version/correction level, it will lead to an error.

        -cl <CHANGELIST>
            Sets a specific change list you want to use.

        -p4u <P4USER>
            Sets the perfoce user to be used.

        -p4c <P4CLIENT>
            Sets the perforce client to be used.

        -p4r <P4ROOT>
            Sets the perforce root to be used.

        -prod
            Initiates a productive make by calling 'NACHT.ALL'.

        -m <MAKE_TARGET>
            Makes a specific target by calling 'imf <MAKE_TARGET>' after
            finishing the sync.

        -cfg <CONFIG_FILE>
            Uses a specific configuration preset.

        -s or -scratch
            Initiates a scratch make. If you've used the -m option to
            compile a single target, a -U will be appended to the imf
            command.

        -c or -clear
            Clears the generated environment again after make.

        -rm or -remove
            Just removes the environment without syncing/making etc.

        -b or -batch
            Start program in batch mode. This option prohibits any
            interactive action by users - if any option is missing,
            it causes the program to fail.

        For more detailed information, please use 'perldoc LCEnv.pl'.

USAGE_END

    exit(1);
}

